import re
import imp
from matuamod_serializer.dto.dto import DTO_TYPE, DTO
from ..parse_arguments import parse_json_arguments
from types import CodeType, FunctionType, ModuleType


class JSON_Parser():

    _arg_dict = parse_json_arguments


    def _get_arg_dict(self) -> dict:
        return self._arg_dict


    def _change_arg_list(self, arg_type: tuple, arg_list: list, get_value: bool = False) -> list:
        if len(arg_list) > 0:
            if arg_type in arg_list:
                arg_list.pop(0)
            return(arg_list if get_value == False else (arg_list, arg_type))


    def _change_arg_list_fast(self, arg_type: tuple, arg_list: list, count: int) -> list:

        for i in range(count):
            arg_list = self._change_arg_list(  # import this
                self._get_first_arg(arg_list), arg_list)
        return arg_list


    def _get_parse_arg(self, json_str: str) -> list:
        json_str = json_str.replace(" ", "")
        arg_dict = self._get_arg_dict()
        arguments = []

        while len(json_str) > 0:

            for argument in arg_dict.parse_arg.items():
                try:
                    expr = re.match(argument[1], json_str)
                    if expr.start() == 0:
                        json_str = json_str[0: expr.start():] + \
                            json_str[expr.end()::]
                        # print("Deleted argument is : " + str(expr.group()))
                        if argument[0] == arg_dict.number:
                            number = float(expr.group()) if "." in expr.group() else int(
                                expr.group())
                            arguments.append((argument[0], number))
                        elif argument[0] == arg_dict.bool_arg:
                            boolean = True if expr.group() == "true" else False
                            arguments.append((argument[0], boolean))
                        elif argument[0] == arg_dict.str_arg:
                            arguments.append(
                                (argument[0], expr.group().replace('"', '')))
                        elif argument[0] == arg_dict.none:
                            arguments.append((argument[0], None))
                        else:
                            arguments.append((argument[0],))
                except:
                    None
        return arguments


    def _get_first_arg(self, arg_list: dict) -> any:
        return (arg_list[0] if len(arg_list) > 0 else None)


    def _parse_prim_types(self, arg_list: list) -> any:
        res_prim = None
        arg_dict = self._get_arg_dict()
        parse_prim = self._get_first_arg(arg_list)
        prim_parse_tuple = (arg_dict.number, arg_dict.str_arg,
                            arg_dict.bool_arg, arg_dict.none)
        if parse_prim[0] in prim_parse_tuple:
            result = parse_prim[1]
            # if parse_prim[0] in arg_dict.str_arg:
            #     if result.isdigit():
            #         result = bytes.fromhex(result)
            arg_list = self._change_arg_list(parse_prim, arg_list)
            return result


    def _parse_list_tuple(self, arg_list: list) -> list:
        res_list = []
        arg_dict = self._get_arg_dict()
        parse_list_item = self._get_first_arg(arg_list)
        if parse_list_item[0] in arg_dict.right_bracket:
            arg_list = self._change_arg_list(parse_list_item, arg_list)

            while self._get_first_arg(arg_list)[0] != arg_dict.left_bracket:
                if self._get_first_arg(arg_list)[0] == arg_dict.comma:
                    arg_list = self._change_arg_list(
                        self._get_first_arg(arg_list), arg_list)
                res_list.append(self._make_parse(arg_list))

            arg_list = self._change_arg_list(
                self._get_first_arg(arg_list), arg_list)
        return (arg_list, res_list)


    def _get_dto_name(self, arg_list: list) -> str:
        dto_names = ('dict', 'func', 'type',
                     'code', 'module', 'obj')

        for count in range(4):
            argument = self._get_first_arg(arg_list)
            if argument[0] == 'str' and argument[1] in dto_names:
                dto_name = argument[1]
            if argument[0] != self._arg_dict.left_brace:
                arg_list = self._change_arg_list(argument, arg_list)
            else:
                return (arg_list, dto_name)
        argument = self._get_first_arg(arg_list)
        return (arg_list, dto_name)


    def _parse_dto(self, arg_list: list) -> any:
        arg_list = self._change_arg_list(
            self._get_first_arg(arg_list), arg_list)
        arg_list, dto_type = self._get_dto_name(arg_list)
        if dto_type == DTO_TYPE.dict:
            arg_list, result = self._parse_dict(arg_list)
        elif dto_type == DTO_TYPE.func:
            arg_list, result = self._parse_func(arg_list)
        elif dto_type == DTO_TYPE.class_type:
            arg_list, result = self._parse_class(arg_list)
        elif dto_type == DTO_TYPE.obj_type:
            arg_list, result = self._parse_obj(arg_list)
        elif dto_type == DTO_TYPE.module:
            arg_list, result = self._parse_module(arg_list)
        return (arg_list, result)


    def _parse_dict(self, arg_list: list) -> dict:
        res_dict = {}

        while len(arg_list) != 0 and self._get_first_arg(arg_list)[0] != self._arg_dict.left_brace:
            # print(self._get_first_arg(arg_list)[0], self._get_first_arg(arg_list)[1])
            if len(self._get_first_arg(arg_list)) == 2:
                arg_list, dict_key = self._change_arg_list(
                    self._get_first_arg(arg_list), arg_list, get_value=True)
                key = self._make_parse([dict_key])
                arg_list = self._change_arg_list(
                    self._get_first_arg(arg_list), arg_list)
                if self._get_first_arg(arg_list)[0] != self._arg_dict.right_bracket\
                        and self._get_first_arg(arg_list)[0] != self._arg_dict.right_brace:
                    arg_list, dict_value = self._change_arg_list(
                        self._get_first_arg(arg_list), arg_list, get_value=True)
                    value = self._make_parse([dict_value])
                else:
                    arg_list, value = self._make_parse(arg_list)
                if len(arg_list) != 0:
                    if self._get_first_arg(arg_list)[0] == self._arg_dict.comma:
                        arg_list = self._change_arg_list(
                            self._get_first_arg(arg_list), arg_list)
                res_dict[key] = value
            elif self._get_first_arg(arg_list)[0] == self._arg_dict.comma:
                arg_list = self._change_arg_list(
                    self._get_first_arg(arg_list), arg_list)

        while len(arg_list) >= 1 and self._get_first_arg(arg_list)[0] == self._arg_dict.left_brace:
            arg_list = self._change_arg_list(
                self._get_first_arg(arg_list), arg_list)
        if len(arg_list) == 1:
            arg_list = self._change_arg_list(
                self._get_first_arg(arg_list), arg_list)
        return (arg_list, res_dict)


    def _parse_func_name(self, arg_list: list) -> str:
        arg_list = self._change_arg_list_fast(
            self._get_first_arg(arg_list), arg_list, 2)
        arg_list, tuple_name = self._change_arg_list(
            self._get_first_arg(arg_list), arg_list, get_value=True)
        arg_list = self._change_arg_list_fast(
            self._get_first_arg(arg_list), arg_list, 1)
        func_name = self._make_parse([tuple_name])
        return (arg_list, func_name)


    def _parse_func_globals(self, arg_list: list) -> dict:
        arg_list = self._change_arg_list_fast(
            self._get_first_arg(arg_list), arg_list, 2)
        arg_list, res_dict = self._make_parse(arg_list)
        return (arg_list, res_dict)


    def _parse_func_code(self, arg_list: list) -> CodeType:
        arg_list = self._change_arg_list_fast(
            self._get_first_arg(arg_list), arg_list, 10)
        arg_list, code_dict = self._make_parse(arg_list)

        res_code = CodeType(
            code_dict["co_argcount"], code_dict["co_posonlyargcount"],
            code_dict["co_kwonlyargcount"], code_dict["co_nlocals"],
            code_dict["co_stacksize"], code_dict["co_flags"],
            bytes.fromhex(code_dict["co_code"]), tuple(code_dict["co_consts"]),
            tuple(code_dict["co_names"]), tuple(code_dict["co_varnames"]),
            code_dict["co_filename"], code_dict["co_name"],
            code_dict["co_firstlineno"], bytes.fromhex(code_dict["co_lnotab"]),
            tuple(code_dict["co_freevars"]), tuple(code_dict["co_cellvars"]),
        )
        return (arg_list, res_code)


    def _parse_func(self, arg_list: list) -> any:
        argument = self._get_first_arg(arg_list)
        if argument[1] == DTO.name:
            arg_list, func_name = self._parse_func_name(arg_list)
            # print(func_name)
            arg_list, func_globals = self._parse_func_globals(arg_list)
            # print(func_globals)
            arg_list, func_code = self._parse_func_code(arg_list)
            # print(func_code)
            arg_list = self._change_arg_list_fast(
                self._get_first_arg(arg_list), arg_list, 3)
            arg_list, buf_closure = self._change_arg_list(
                self._get_first_arg(arg_list), arg_list, get_value=True)
            arg_list = self._change_arg_list_fast(
                self._get_first_arg(arg_list), arg_list, 1)
            func_closure = self._make_parse(buf_closure)
        res_func = FunctionType(func_code, func_globals,
                                func_name, func_closure)
        res_func.__globals__["__builtins__"] = __import__("builtins")
        return (arg_list, res_func)


    def _parse_name(self, arg_list: list) -> str:
        arg_list = self._change_arg_list_fast(
            self._get_first_arg(arg_list), arg_list, 2)
        arg_list, tuple_name = self._change_arg_list(
            self._get_first_arg(arg_list), arg_list, get_value=True)
        name = self._make_parse([tuple_name])
        arg_list = self._change_arg_list_fast(
            self._get_first_arg(arg_list), arg_list, 1)
        return (arg_list, name)


    def _parse_fields(self, arg_list: list) -> dict:
        arg_list, argument = self._change_arg_list(
            self._get_first_arg(arg_list), arg_list, get_value=True)
        if argument[1] == "fields":
            arg_list = self._change_arg_list(
                self._get_first_arg(arg_list), arg_list)
            arg_list, fields_dict = self._make_parse(arg_list)
        return (arg_list, fields_dict)


    def _parse_class(self, arg_list: list) -> type:
        arg_list, class_name = self._parse_name(arg_list)
        arg_list, class_fields = self._parse_fields(arg_list)
        class_bases = (object,)
        if "__bases__" in class_fields:
            class_bases = tuple(class_fields["__bases__"])
        return (arg_list, type(class_name, class_bases, class_fields))


    def _parse_obj_base_class(self, arg_list: list) -> type:
        arg_list = self._change_arg_list_fast(self._get_first_arg, arg_list, 2)
        arg_list, base_dict = self._make_parse(arg_list)
        return (arg_list, base_dict)


    def _parse_obj_fields(self, arg_list: list):
        if self._get_first_arg(arg_list)[0] == self._arg_dict.comma:
            arg_list = self._change_arg_list_fast(self._get_first_arg(arg_list), arg_list, 3)
        arg_list, fields_dict = self._make_parse(arg_list)
        return (arg_list, fields_dict)


    def _parse_obj(self, arg_list: list) -> object:
        arg_list, base_dict = self._parse_obj_base_class(arg_list)
        arg_list, fields_dict = self._parse_obj_fields(arg_list)

        class_init = base_dict.__init__
        if callable(class_init):
            if class_init.__class__.__name__ == "function":
                delattr(base_dict, "__init__")
        obj = base_dict()
        obj.__init__ = class_init
        obj.__dict__ = fields_dict
        return (arg_list, obj)


    def _parse_module(self, arg_list: list) -> ModuleType:
        arg_list, module_name = self._parse_name(arg_list)
        arg_list, fields_dict = self._parse_fields(arg_list)

        module = None
        if fields_dict == None:
            module = __import__(module_name)
        else:
            module = imp.new_module(module_name)
            for field in fields_dict.items():
                setattr(module,field[0],field[1])
        return (arg_list, module)


    def _make_parse(self, arg_list: list) -> any:
        argument = self._get_first_arg(arg_list)
        if argument[0] == self._arg_dict.right_brace:
            arg_list, result = self._parse_dto(arg_list)
        elif argument[0] == self._arg_dict.right_bracket:
            arg_list, result = self._parse_list_tuple(arg_list)
        else:
            result = self._parse_prim_types(arg_list)
        return ((arg_list, result) if argument[0] == self._arg_dict.right_brace or
                argument[0] == self._arg_dict.right_bracket else result)


    def _parse(self, json_str: str) -> any:
        arg_list = self._get_parse_arg(json_str)
        result = self._make_parse(arg_list)
        return result



right_brace = "right_brace"
left_brace = "left_brace"
right_bracket = "right_bracket"
left_bracket = "left_bracket"
comma = "comma"
colon = "colon"
none = "null"
number = "number"
str_arg = "str"
bool_arg = "bool"
eof = "eof"


parse_arg = {
    right_brace: r"{",
    left_brace: r"}",
    right_bracket: "\\[",
    left_bracket: "\\]",
    comma: r",",
    colon: r":",
    none: r"null",
    number: r'([0-9]*[.])?[0-9]+',
    str_arg: r'"[^"]*"',
    bool_arg: r'^(?:tru|fals)e',
    eof: r"eof"
}
    




from types import NoneType, ModuleType
import toml
from matuamod_serializer.parser.toml_parser.toml_parser import TOML_Parser
from ..base_serializer.base_serializer import BaseSerializer
from matuamod_serializer.dto.dto import DTO, DTO_TYPE
from ..dependencies import dependencies
from matuamod_serializer.parser.toml_parser.toml_parser import TOML_Parser
import inspect


class TOML_Serializer(BaseSerializer):

    _str = ""
    _toml_parser = None


    def __init__(self):
        super().__init__()
        self._toml_parser = TOML_Parser()

    
    def dump(self, obj: any, file_path: str):
        with open(file_path, "w") as file:
            res_str = self.dumps(obj)
            file.write(res_str)


    def dumps(self, obj: any) -> str:
        return self._make_serialize(obj)


    def load(self, file_path: str) -> any:
        with open(file_path, 'r') as file:
            toml_str = file.read()
            return self.loads(toml_str)


    def loads(self, s: str) -> any:
        obj_dict = toml.loads(s)
        return self._toml_parser._make_parse(obj_dict)


    def _add(self, obj: any) -> str:
        self._str = toml.dumps(obj)


    def _make_serialize(self, obj: any) -> str:
        primitive_types = (int, float, bool, str, bytes, NoneType) 
        if type(obj) in primitive_types or obj == None:
            res_dict = self._check_is_first(obj)
        else:
            res_dict = self._inspect(obj)
        self._add(res_dict)
        return (self._str)


    def _check_is_first(self, prim_obj: any) -> dict:
        prim_dict = {}
        if type(prim_obj) in (int, float):
            prim_dict[f'{DTO_TYPE.number}'] = prim_obj
        elif type(prim_obj) == str:
            prim_dict[f'{DTO_TYPE.str}'] = prim_obj
        elif type(prim_obj) == bytes:
            prim_dict[f'{DTO_TYPE.bytes}'] = prim_obj.hex()
        elif type(prim_obj) == bool:   
            prim_dict[f'{DTO_TYPE.bool}'] = prim_obj
        elif type(prim_obj) == NoneType:
            prim_dict[f'{DTO_TYPE.none_type}'] = "null"
        return prim_dict

    
    def _inspect(self, obj: any):
        primitive_types = (int, float, bool, str, bytes, NoneType) 
        if type(obj) in primitive_types or obj == None:
            result = self._inspect_prim_types(obj)
        elif type(obj) in (list, tuple):
            result = self._inspect_list(obj)
        elif type(obj) == dict:
            result = self._inspect_dict(obj)
        elif inspect.isfunction(obj):
            result = self._inspect_func(obj)
        elif inspect.isclass(obj):
            result = self._inspect_class(obj)
        elif type(obj) == ModuleType:
            result = self._inspect_obj_module(obj)
        elif isinstance(obj, object):
            result = self._inspect_obj(obj)
        return result


    def _inspect_prim_types(self, prim_obj: any) -> any:
        if type(prim_obj) == bytes:
            prim_obj = prim_obj.hex()
        return (prim_obj if type(prim_obj) != NoneType else "null")


    def _inspect_list(self, list_obj: list) -> dict:
        list_dict = {}
        list_dict[f'{DTO.dto_type}'] = f'{DTO_TYPE.list}'        
        if list_obj == []:
            return list_dict
        else:
            for i, member in enumerate(list_obj):
                list_dict[f'item_{i}'] = self._inspect(member)
            return list_dict


    def _inspect_dict(self, dict_obj: dict) -> dict:
        res_dict = {}
        res_dict[f'{DTO.dto_type}'] = f'{DTO_TYPE.dict}'
        if dict_obj == {}:
            return res_dict
        else:
            for item in dict_obj.items():
                res_dict[item[0]] = self._inspect(item[1])
            return res_dict
 
    
    def _inspect_func(self, func_obj) -> dict:
        func_dict = {}
        func_dict[f'{DTO.dto_type}'] = f'{DTO_TYPE.func}'
        func_dict[f'{DTO.name}'] = self._inspect(func_obj.__name__)
        globals_dict = dependencies.get_globals(func_obj)
        func_dict[f'{DTO.global_types}'] = self._inspect(globals_dict)
        code_dict = dependencies.get_code_fields(func_obj.__code__)
        func_dict[f'{DTO.code}'] = self._inspect(code_dict)
        func_dict[f'{DTO.closure}'] = self._inspect(func_obj.__closure__)
        return func_dict


    def _inspect_class(self, class_obj: type) -> dict:
        class_dict = {}
        class_dict[f'{DTO.dto_type}'] = f'{DTO_TYPE.class_type}'
        class_dict[f'{DTO.name}'] = self._inspect(class_obj.__name__)
        fields_dict = dependencies.get_class_fields(class_obj)
        class_dict[f'{DTO.fields}'] = self._inspect(fields_dict)
        return class_dict

    
    def _inspect_obj(self, obj):
        obj_dict = {}
        obj_dict[f'{DTO.dto_type}'] = f'{DTO_TYPE.obj_type}'
        obj_dict[f'{DTO.base_class}'] = self._inspect(obj.__class__)
        obj_dict[f'{DTO.fields}'] = self._inspect(obj.__dict__)
        return obj_dict


    def _inspect_obj_module(self, obj_module):
        module_dict = {}
        module_dict[f'{DTO.dto_type}'] = f'{DTO_TYPE.module}'
        module_dict[f'{DTO.name}'] = self._inspect(obj_module.__name__)
        if dependencies.is_std_lib_module(obj_module):
            module_dict[f'{DTO.fields}'] = self._inspect(None)
        else:
            module_fields = dependencies.get_module_fields(obj_module)
            module_dict[f'{DTO.fields}'] = self._inspect(module_fields)
        return module_dict

        


from matuamod_serializer.dto.dto import DTO, DTO_TYPE
from types import NoneType, FunctionType, ModuleType, CodeType
import yaml
import imp


class YAML_Parser():

    def _make_parse(self, yaml_str: str) -> any:
        obj = yaml.full_load(yaml_str)
        res_obj = self._parse_types(obj)
        return res_obj


    def _parse_types(self, obj: any) -> any:
        prim_types = (int, float, bool, str, bytes, NoneType)
        if type(obj) in prim_types:
            res_obj = self._parse_prim_type(obj)
        elif type(obj) == list:
            res_obj = self._parse_list_type(obj)
        else:
            res_obj = self._parse_dto(obj)
        return res_obj


    def _parse_prim_type(self, prim_obj: any) -> any:
        return prim_obj

    
    def _parse_list_type(self, list_obj: list) -> list:
        res_list = []
        for item in list_obj:
            res_list.append(self._parse_types(item))
        return res_list


    def _parse_dto(self, dto_obj: any) -> any:
        if dto_obj[DTO.dto_type] == DTO_TYPE.dict:
            obj = self._parse_dict_type(dto_obj)
        elif dto_obj[DTO.dto_type] == DTO_TYPE.func:
            obj = self._parse_func_type(dto_obj)
        elif dto_obj[DTO.dto_type] == DTO_TYPE.class_type:
            obj = self._parse_class_type(dto_obj)
        elif dto_obj[DTO.dto_type] == DTO_TYPE.obj_type:
            obj = self._parse_obj_type(dto_obj)
        elif dto_obj[DTO.dto_type] == DTO_TYPE.module:
            obj = self._parse_module(dto_obj)
        return obj

    
    def _parse_dict_type(self, dict_obj: dict) -> dict:
        res_dict = {}
        dict_obj.pop(DTO.dto_type)
        for item in dict_obj.items():
            res_dict[self._parse_types(item[0])] = self._parse_types(item[1])
        return res_dict


    def _parse_func_type(self, func_obj: dict) -> FunctionType:
        func_obj.pop(DTO.dto_type)
        func_name = self._parse_types(func_obj[DTO.name])
        func_globals = self._parse_types(func_obj[DTO.global_types])
        code_dict = self._parse_types(func_obj[DTO.code])
        func_closure = self._parse_types(func_obj[DTO.closure])

        func_code = CodeType(
            code_dict["co_argcount"], code_dict["co_posonlyargcount"],
            code_dict["co_kwonlyargcount"], code_dict["co_nlocals"],
            code_dict["co_stacksize"], code_dict["co_flags"],
            code_dict["co_code"], tuple(code_dict["co_consts"]),
            tuple(code_dict["co_names"]), tuple(code_dict["co_varnames"]),
            code_dict["co_filename"], code_dict["co_name"],
            code_dict["co_firstlineno"], code_dict["co_lnotab"],
            tuple(code_dict["co_freevars"]), tuple(code_dict["co_cellvars"]),
        )

        res_func = FunctionType(func_code, func_globals,
                                func_name, func_closure)
        res_func.__globals__["__builtins__"] = __import__("builtins")
        return res_func


    def _parse_class_type(self, class_obj: dict) -> type:
        class_obj.pop(DTO.dto_type)
        class_name = self._parse_types(class_obj[DTO.name])
        class_fields = self._parse_types(class_obj[DTO.fields])
        class_bases = (object,)
        if "__bases__" in class_fields:
            class_bases = tuple(class_fields["__bases__"])
        res_class = type(class_name, class_bases, class_fields)
        return res_class


    def _parse_obj_type(self, obj_type: dict) -> any:
        obj_type.pop(DTO.dto_type)
        base_class = self._parse_types(obj_type[DTO.base_class])
        obj_fields = self._parse_types(obj_type[DTO.fields])

        class_init = base_class.__init__
        if callable(class_init):
            if class_init.__class__.__name__ == 'function':
                delattr(base_class, "__init__")
        obj = base_class()
        obj.__init__ = class_init
        obj.__dict__ = obj_fields
        return obj


    def _parse_module(self, module_obj: dict) -> ModuleType:
        module_name = self._parse_types(module_obj[DTO.name])
        fields_dict = self._parse_types(module_obj[DTO.fields])
        
        module = None
        if fields_dict == None:
            module = __import__(module_name)
        else:
            module = imp.new_module(module_name)
            for field in fields_dict.items():
                setattr(module,field[0],field[1])
        return module



from abc import ABC, abstractmethod

class BaseSerializer(ABC):

    @abstractmethod
    def dump(self, obj : any, file_path : str):
        pass

    @abstractmethod
    def dumps(self, obj : any) -> str:
        pass

    @abstractmethod
    def load(self, file_path : str) -> any:
        pass

    @abstractmethod
    def loads(self, data : str) -> any:
        pass


from .serializer.json_serializer.json_serializer import JSON_Serializer
from .parser.json_parser.json_parser import JSON_Parser
from .serializer.toml_serializer.toml_serializer import TOML_Serializer
from .parser.toml_parser.toml_parser import TOML_Parser
from .serializer.yaml_serializer.yaml_serilizer import YAML_Serializer
from .parser.yaml_parser.yaml_parser import YAML_Parser
from .dto.dto import DTO, DTO_TYPE
from .converters.converters import Converter
from .create_serializer.create_serializer import create_serializer


import sys
from .converters.converters import Converter


def main():
    converter = Converter()
    args = converter.parse_args()
    converter.get_args(args)


if __name__ == "__main__":
    main()


from modulefinder import Module
from matuamod_serializer.parser.json_parser.json_parser import JSON_Parser
from ..base_serializer.base_serializer import BaseSerializer
from matuamod_serializer.dto.dto import DTO, DTO_TYPE
from ..dependencies import dependencies
import inspect
from types import ModuleType, NoneType


class JSON_Serializer(BaseSerializer):

    _str = ""
    _json_parser = None
 

    def __init__(self):
        super().__init__()
        self._json_parser = JSON_Parser()


    def dump(self, obj: any, file_path: str):
        file = open(file_path, "w")
        _str = self.dumps(obj)
        file.write(_str)
        file.close()


    def dumps(self, obj: any) -> str:
        self._str = ""
        self._inspect(obj)
        return self._str


    def load(self, file_path: str) -> any:
        obj = None
        file = open(file_path, "r")
        _str = file.read()
        obj = self.loads(_str)
        return obj


    def loads(self, s: str) -> any:
        return self._json_parser._parse(s)


    def _add(self, type_str: str):
        self._str += type_str


    def _inspect(self, obj):
        primitive_types = (int, float, bool, str, bytes)
        if type(obj) in primitive_types:
            self._inspect_primitive_type(obj)
        elif type(obj) in (tuple, list):
            self._inspect_list_tuple_type(obj)
        elif obj == None:
            self._add("null")
        else:
            self._add("{")
            if type(obj) == dict:
                self._inspect_dict_type(obj)
            elif inspect.isfunction(obj):
                self._inspect_func_type(obj)
            elif inspect.isclass(obj):
                self._inspect_class_type(obj)
            elif type(obj) == ModuleType:
                self._inspect_obj_module(obj)
            elif isinstance(obj, object):
                self._inspect_obj_type(obj)
            self._add("}")


    def _inspect_primitive_type(self, prim_obj):
        prim_obj_type = type(prim_obj)
        if prim_obj_type in (int, float):
            self._add(f'{prim_obj}')
        elif prim_obj_type == bool:
            if prim_obj:
                bool_value = "true"
            else:
                bool_value = "false"
            self._add(f'{bool_value}')
        elif prim_obj_type == str:
            self._add(f'"{prim_obj}"')
        elif prim_obj_type == bytes:
            hexademical = prim_obj.hex()
            self._add(f'"{hexademical}"')


    def _inspect_list_tuple_type(self, obj):
        if(len(obj)) == 0:
            self._add('[]')
        else:
            self._add('[')
            for i, part_obj in enumerate(obj):
                if i != 0:
                    self._add(',')
                self._inspect(part_obj)
            self._add(']')


    def _inspect_dict_type(self, dict_obj: dict):
        self._add(f'"{DTO.dto_type}": "{DTO_TYPE.dict}"')
        if len(dict_obj) >= 1:
            self._add(",")
            is_first_el = True
            i = 0

        for key, value in dict_obj.items():
            if is_first_el != True:
                self._add(',')
            self._inspect(key)
            self._add(': ')
            self._inspect(value)
            is_first_el = False


    def _get_code(self, func_obj):
        code_obj = func_obj.__code__
        self._add(f'"{DTO.code}": ')
        self._add('{')
        self._add(f'"{DTO.dto_type}": "{DTO_TYPE.code}",')
        self._add(f'"{DTO.fields}": ')
        curr_code_dict = dependencies.get_code_fields(code_obj)
        self._inspect(curr_code_dict)
        self._add('}')


    def _inspect_func_type(self, func_obj):
        self._add(f'"{DTO.dto_type}": "{DTO_TYPE.func}",')
        self._add(f'"{DTO.name}": "{func_obj.__name__}",')
        self._add(f'"{DTO.global_types}": ')
        curr_globals_dict = dependencies.get_globals(func_obj)
        self._inspect(curr_globals_dict)
        self._add(',')
        self._get_code(func_obj)
        self._add(',')
        self._add(f'"{DTO.closure}": "{func_obj.__closure__}"')


    def _inspect_class_type(self, class_obj):
        fields_dict = dependencies.get_class_fields(class_obj)
        self._add(f'"{DTO.dto_type}": "{DTO_TYPE.class_type}",')
        self._add(f'"{DTO.name}": "{class_obj.__name__}",')
        self._add(f'"{DTO.fields}": ')
        self._inspect(fields_dict)


    def _inspect_obj_type(self, obj):
        self._add(f'"{DTO.dto_type}": "{DTO_TYPE.obj_type}",')
        self._add(f'"{DTO.base_class}": ')
        self._inspect(obj.__class__)
        self._add(",")
        self._add(f'"{DTO.fields}": ')
        self._inspect(obj.__dict__)


    def _inspect_obj_module(self, obj_module):
        self._add(f'"{DTO.dto_type}": "{DTO_TYPE.module}",')
        self._add(f'"{DTO.name}": "{obj_module.__name__}",')
        self._add(f'"{DTO.fields}": ')
        if dependencies.is_std_lib_module(obj_module):
            self._inspect(None)
        else:
            module_fields = dependencies.get_module_fields(obj_module)
            self._inspect(module_fields)



from ..serializer.base_serializer.base_serializer import BaseSerializer
from ..serializer.json_serializer.json_serializer import JSON_Serializer
from ..serializer.toml_serializer.toml_serializer import TOML_Serializer
from ..serializer.yaml_serializer.yaml_serilizer import YAML_Serializer


SERIALIZERS = {
    "json": JSON_Serializer,
    "toml": TOML_Serializer,
    "yaml": YAML_Serializer
}

def create_serializer(serializer_name: str) -> BaseSerializer:
    if serializer_name in SERIALIZERS:
        return SERIALIZERS[serializer_name]()
    else:
        return None




from .json_serializer.json_serializer import JSON_Serializer
from .toml_serializer.toml_serializer import TOML_Serializer
from .yaml_serializer.yaml_serilizer import YAML_Serializer
from .base_serializer.base_serializer import BaseSerializer
from .dependencies import dependencies


from dataclasses import dataclass


@dataclass
class DTO():
    dto_type = "dto_type"
    name = "name"
    code = "code"
    fields = "fields"
    global_types = "globals"
    closure = "closure"
    base_class = "base_class"

@dataclass
class DTO_TYPE():
    obj_type = "obj"
    func = "func"
    code = "code"
    module = "module"
    list = "list"
    dict = "dict"
    class_type = "type"
    number = "number"
    str = "str"
    bytes = "bytes"
    bool = "bool"
    none_type = "none"
    


from setuptools import find_packages, setup

setup(
    name='matuamod_serializer',
    packages=['matuamod_serializer'],
    version='0.2.2',
    description='My json, toml, yaml serializer/parser library',
    author='matuamod',
    author_email='matua.models2003@gmail.com',
    url='https://github.com/matuamod/ISP-2022-053504/',
    license='MIT',
)



import inspect
from matuamod_serializer.dto.dto import DTO, DTO_TYPE
from types import NoneType, WrapperDescriptorType, MethodDescriptorType, BuiltinFunctionType, MappingProxyType, GetSetDescriptorType, ModuleType
import sys
import imp

def get_globals(func_obj) -> dict:
    globals_type = func_obj.__globals__
    curr_globals_dict = dict()
    code_obj = func_obj.__code__

    for key, value in globals_type.items():
        if key in code_obj.co_names:
            curr_globals_dict.update({key: value})
    return curr_globals_dict


def get_code_fields(code_obj) -> dict:
    curr_code_dict = dict()
    variables = (
        "co_nlocals", "co_argcount",
        "co_varnames", "co_names",
        "co_cellvars", "co_freevars",
        "co_posonlyargcount", "co_kwonlyargcount",
        "co_firstlineno", "co_lnotab",
        "co_stacksize", "co_code", "co_name",
        "co_consts", "co_flags", "co_filename"
    )

    for member in inspect.getmembers(code_obj):
        if member[0] in variables:
            curr_code_dict.update({member[0]: member[1]})
    return curr_code_dict


def get_class_fields(class_obj) -> dict:
    fields_dict = dict()
    if class_obj == type:
        fields_dict["__bases__"] = []
    else:
        members = inspect.getmembers(class_obj)

        for member in members:
            if type(member[1]) not in (
                WrapperDescriptorType,
                MethodDescriptorType,
                BuiltinFunctionType,
                MappingProxyType,
                GetSetDescriptorType
            ) and member[0] not in (
                "__mro__", "__base__", "__basicsize__",
                "__class__", "__dictoffset__", "__name__",
                "__qualname__", "__text_signature__", "__itemsize__",
                "__flags__", "__weakrefoffset__"
            ):
                fields_dict[member[0]] = member[1]
    return fields_dict


def get_module_fields(obj_module: ModuleType) -> dict:
    module_fields = {}
    for member in inspect.getmembers(obj_module):
        if not member[0].startswith("__"):
            module_fields[member[0]] = member[1]
    return module_fields


def is_std_lib_module(obj_module: ModuleType):
    libs_path = sys.path[2]
    module_path = imp.find_module(obj_module.__name__)[1]
    if obj_module.__name__ in sys.builtin_module_names:
        return True
    elif libs_path in module_path:
        return True
    elif "site-packages" in module_path:
        return True
    else:
        return False


from matuamod_serializer.dto.dto import DTO, DTO_TYPE
import yaml
import inspect
from types import NoneType, FunctionType, ModuleType
from ..dependencies import dependencies
from ..base_serializer.base_serializer import BaseSerializer
from matuamod_serializer.parser.yaml_parser.yaml_parser import YAML_Parser 


class YAML_Serializer(BaseSerializer):

    _yaml_parser = None

    def __init__(self):
        super().__init__()
        self._yaml_parser = YAML_Parser()

    
    def dump(self, obj: any, file_path: str):
        with open(file_path, 'w') as file:
            res_str = self.dumps(obj)
            file.write(res_str)

    
    def dumps(self, obj: any):
        return self._make_serialize(obj)


    def load(self, file_path: str):
        with open(file_path, 'r') as file:
            res_str = file.read()
            return self.loads(res_str)


    def loads(self, s: str):
        return self._yaml_parser._make_parse(s)


    def _inspect_types(self, obj: any) -> any:
        prim_types = (int, float, bool, str, bytes, NoneType)
        if type(obj) in prim_types:
            res_obj = self._inspect_prim_type(obj)
        elif type(obj) in (list, tuple):
            res_obj = self._inspect_list(obj)
        elif type(obj) == dict:
            res_obj = self._inspect_dict(obj)
        elif type(obj) == FunctionType:
            res_obj = self._inspect_func(obj)
        elif type(obj) == type:
            res_obj = self._inspect_class(obj)
        elif type(obj) == ModuleType:
            res_obj = self._inspect_module(obj)
        elif isinstance(obj, object):
            res_obj = self._inspect_obj(obj)
        return res_obj


    def _inspect_prim_type(self, prim_obj: any) -> any:
        return prim_obj


    def _inspect_list(self, list_obj: list) -> list: 
        res_list = []
        for item in list_obj:
            res_list.append(self._inspect_types(item))
        return res_list


    def _inspect_dict(self, dict_obj: dict) -> dict:
        res_dict = {}
        res_dict[f'{DTO.dto_type}'] = f'{DTO_TYPE.dict}'
        for item in dict_obj.items():
            res_dict[self._inspect_types(item[0])] = self._inspect_types(item[1])
        return res_dict


    def _inspect_func(self, func_obj: FunctionType) -> dict:
        func_dict = {}
        func_dict[f'{DTO.dto_type}'] = f'{DTO_TYPE.func}'
        func_dict[f'{DTO.name}'] = self._inspect_types(func_obj.__name__)
        globals_dict = dependencies.get_globals(func_obj)
        func_dict[f'{DTO.global_types}'] = self._inspect_types(globals_dict)
        func_code = dependencies.get_code_fields(func_obj.__code__)
        func_dict[f'{DTO.code}'] = self._inspect_types(func_code)
        func_dict[f'{DTO.closure}'] = self._inspect_types(func_obj.__closure__)
        return func_dict


    def _inspect_class(self, class_obj: type) -> dict:
        class_dict = {}
        class_dict[f'{DTO.dto_type}'] = f'{DTO_TYPE.class_type}'
        class_dict[f'{DTO.name}'] = self._inspect_types(class_obj.__name__)
        class_fields = dependencies.get_class_fields(class_obj)
        class_dict[f'{DTO.fields}'] = self._inspect_types(class_fields)
        return class_dict


    def _inspect_obj(self, obj_type: any) -> dict:
        obj_dict = {}
        obj_dict[f'{DTO.dto_type}'] = f'{DTO_TYPE.obj_type}'
        obj_dict[f'{DTO.base_class}'] = self._inspect_types(obj_type.__class__)
        obj_dict[f'{DTO.fields}'] = self._inspect_types(obj_type.__dict__)
        return obj_dict


    def _inspect_module(self, module_obj: ModuleType) -> dict:
        module_dict = {}
        module_dict[f'{DTO.dto_type}'] = f'{DTO_TYPE.module}'
        module_dict[f'{DTO.name}'] = self._inspect_types(module_obj.__name__)
        if dependencies.is_std_lib_module(module_obj): 
            module_dict[f'{DTO.fields}'] = self._inspect_types(None)
        else:
            module_fields = dependencies.get_module_fields(module_obj)
            module_dict[f'{DTO.fields}'] = self._inspect_types(module_fields)
        return module_dict

    
    def _make_serialize(self, obj: any) -> str:
        res_obj = self._inspect_types(obj)
        return yaml.dump(res_obj, sort_keys=False)
        



from ..create_serializer.create_serializer import create_serializer
import argparse
import sys
import os
import configparser


class Converter():
    
    _parser = None


    def parse_args(self):
        self._parser = argparse.ArgumentParser(
            description="Matuamod's custom serializer"
        )
        self._parser.add_argument(
            "--src_file", type=str, help="file_path for parse"
        )
        self._parser.add_argument(
            "--to_file", type=str, help="file_path to write"
        )
        self._parser.add_argument(
            "--make_config", type=str, help="use only config file and ignore other args"
        )
        return self._parser.parse_args()


    def get_args(self, args):
        src_file = None
        to_file = None
        if args.src_file and args.to_file:
            src_file, to_file = args.src_file, args.to_file
        elif args.make_config:
            print(args.make_config)
            src_file, to_file = self.get_config(args.make_config)
        if src_file and to_file:
            if os.path.exists(src_file) and os.path.isfile(src_file)\
                and os.path.exists(to_file) and os.path.isfile(to_file):
                # Get path of files
                abs_src_path = os.path.abspath(src_file)
                abs_to_path = os.path.abspath(to_file)
                self.make_convert(abs_src_path, abs_to_path)

            else:
                self._parser.error(f'No such files : {src_file} or {to_file}')
        else:
            self.__parser.error(
                f'Invalid --src_file and --to_file args: src_file="{src_file}", to_file="{to_file}"')


    
    def get_config(self, config_path: str) -> tuple:
        config_file = os.path.abspath(config_path)
        if os.path.isfile(config_file) and os.path.exists(config_file):
            config = configparser.RawConfigParser()
            config.read(config_file)
            src_file = config.get("DEFAULT", "src_file")
            to_file = config.get("DEFAULT", "to_file")
            return (src_file.replace('"', ''), to_file.replace('"', ''))
        else:
            self._parser.error(f'No such config file : {config_file}')


    def make_convert(self, src_file: str, to_file: str):
        src_format = os.path.splitext(src_file)[1][1:]
        to_format = os.path.splitext(to_file)[1][1:]
        if src_format != to_format:
            create_src_ser = create_serializer(src_format)
            create_to_ser = create_serializer(to_format)
            if create_src_ser and create_to_ser:
                ser_obj = create_src_ser.load(src_file)
                print(ser_obj)
                create_to_ser.dump(ser_obj, to_file)
        else:
            print("Chosen formats are equal.")
